#        Name : Pushpull_z
# Description : Extrudes a face along Z axis, not along the face normal
#      Author : Didier Bur
#       Usage : select a face, select from the Plugins menu, enter height and here you go
#        Date : 03.Jul.2oo4
#        Type : tool

require 'sketchup.rb'

def line_to_wall

model = Sketchup.active_model
model.start_operation "Line2wall"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end
if not ss.first.kind_of? Sketchup::Edge
  UI.messagebox("Please select a line.")
  return nil
end
if ss.length > 1
UI.messagebox("Please select only one line.")
  return nil
end

prompts = ["Wall width", "Wall height "]
value = [25.cm,250.cm]
results = inputbox prompts, value, "Wall parameters"

return if not results # This means that the user canceld the operation
    width, height = results
 
pts = ss[0].vertices


# copy the selected line
i = 0
pts2 = []

  pts.each do |pt|
    pts2[i] = pt.position + [0 , 0 , results[1]]
    i = i +1
  end
  line_copy = entities.add_line pts2

pts_copy = line_copy.vertices

# create lines between pairs of vertices of each face
i = 0
j = 0

#0.upto(pts.length - 2) do |new_line| 
#  #new_face = entities.add_face(pts[i], pts[i + 1], pts_copy[i + 1], pts_copy[i])
#  new_line = entities.add_line(pts[i], pts_copy[i])
#  i = i + 1
#end    # of upto

# close the inner wall
new_face = entities.add_face(pts[0], pts_copy[0], pts_copy[1], pts[1])
new_face.pushpull width

model.commit_operation
end    # of def


if( not file_loaded?("line2wall.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
    
end

#-----------------------------------------------------------------------------
file_loaded("line2wall.rb")

#        Name : lines2multiwalls
# Description : Creates walls starting from a selection of lines, an arc, a circle, a curve
#      Author : Didier Bur
#       Usage : select lines, an arc, a circle, a curve, select from the Plugins menu, 
#               select wall style, wall height and here you go
#        Date : 06.Oct.2oo4
#        Type : tool


require 'sketchup.rb'

#----------------------------------------------------------------------------- Utilities
def polar(p1, p2, d, angle)
 
  v = p2 - p1
  v.length = d
  translate = Geom::Transformation.translation(v)
  p_temp = p1.transform(translate)
  rotate = Geom::Transformation.rotation( p1, Geom::Vector3d.new(0, 0, 1), angle)
  result = p_temp.transform(rotate)
  return result

end

#----------------------------------------------------------------------------- Main stuff
def line_to_multiwalls

model = Sketchup.active_model
model.start_operation "Lines/curves to multiwalls"
entities = model.active_entities
ss = model.selection
pis2 = 1.5707963267948965

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# Search for existing wall styles in the model
  ad = model.attribute_dictionaries
  mlw_names = []
  mlw_params = []
    # If MLW attribute_dictionnary exists, then get it, else create it
    if ad
      ad_mlw = ad["MLW"]
    end
  # Collect the MLW names, if any
  i = 0
  entities.each do |e|
    #att = e.attribute_dictionary("MLW") 
    if e.attribute_dictionary("MLW")
      mlw_names[i] = e.get_attribute("MLW", "n")
      mlw_params[i] = e.get_attribute("MLW", "p")
      i = i + 1
    end 
  end #of each
  mlw_names.uniq!
  mlw_params.uniq!
  
 # Dialog box: prompts for style and height
  $mlw_name = "" if not $mlw_name
  $height = 250.cm if not $height
  prompts = ["Existing wall styles  ", "Wall height "]
  enums = [mlw_names.join("|")]
  values = [$mlw_name, $height]
  results = inputbox prompts, values, enums, mlw_names.length.to_s + " styles available:"
  return if not results
  $mlw_name, $height = results

  $params =  mlw_params[mlw_names.index($mlw_name)].split(";")

  $layers_mat = []
  $layers_wid = []
  $axis = $params[0]
  $just = $params[1]
  $layer_n = $params[2].to_i
  
  #puts $axis + "\n"
  #puts $just + "\n"
  #puts $layer_n.to_s + "\n"
  
  $layers_mat[0] = $params[3]
  $layers_wid[0] = $params[4].to_f.cm
  $layers_mat[1] = $params[5]
  $layers_wid[1] = $params[6].to_f.cm
  $layers_mat[2] = $params[7]
  $layers_wid[2] = $params[8].to_f.cm
  $layers_mat[3] = $params[9]
  $layers_wid[3] = $params[10].to_f.cm
    
  $layer_wid_1 = $layers_wid[0]
  $layer_wid_2 = $layers_wid[1]
  $layer_wid_3 = $layers_wid[2]
  $layer_wid_4 = $layers_wid[3]
    
  $wall_width = $layer_wid_1 + $layer_wid_2 + $layer_wid_3 + $layer_wid_4
  
  #puts "WW: " + $wall_width.to_s + "\n"
  
  # define a new group
    group = Sketchup.active_model.entities.add_group
    entities = group.entities
    # set an attribute MLW:name
    group.set_attribute "MLW", "n", $mlw_name
    # set an attribute MLW:params
    $params = $axis + ";" + $just + ";" + $layer_n.to_s + ";" + $layers_mat[0] + ";" + $layers_wid[0].to_s + ";" + $layers_mat[1] + ";" + $layers_wid[1].to_s + ";" + $layers_mat[2] + ";" + $layers_wid[2].to_s + ";" + $layers_mat[3] + ";" + $layers_wid[3].to_s + ";"
    group.set_attribute "MLW", "p", $params

# construct the wall profile
# We must first find a vertex that belongs to only one edge (start or end of the selection set)

i = 0
indice = 0

0.upto(ss.length - 1) do |something|
  pts = ss[i].vertices
  v1 = pts[0]
  v2 = pts[1]
  edges1 = pts[0].edges         # Array of edges that use this vertex (first endpoint of current edge)
  edges2 = pts[1].edges         # Array of edges that use this vertex (second endpoint of current edge)
  start_edge= []

  
  if( edges1.length == 1 )      # This means we have a "free" end of edge
    indice = i                   # returns indice of edge in the selection set
  break
  end
  if( edges2.length == 1 )      # This means we have a "free" end of edge
    indice = i                   # returns indice of edge in the selection set
  break
  end
  i = i + 1
end # of upto

# Which end of the edge is the good one ?
pts = ss[indice].vertices
if( pts[0].edges.length == 1 )
  pt1 = pts[0].position
  pt2 = pts[1].position
  vec = pt1.vector_to pt2
  else
  pt1 = pts[1].position
  pt2 = pts[0].position
  vec = pt2.vector_to pt1
end # of if

# Path
j = 0
path = []

0.upto(ss.length - 1) do |something|
  path[j] = ss[j]
  j = j + 1
end

# Draw the wall profile, from left to right
    
# First layer
    vec.length = $wall_width.to_f
    pt1_1l = polar( pt1, pt2, $wall_width.to_f, pis2 )
    pt1_1r = polar( pt1, pt2, ($wall_width.to_f - $layer_wid_1.to_f), pis2 )
    pt2_1l = Geom::Point3d.new(pt1_1l.x, pt1_1l.y, pt1_1l.z + $height)
    pt2_1r = Geom::Point3d.new(pt1_1r.x, pt1_1r.y, pt1_1r.z + $height)
  face = entities.add_face(pt1_1l, pt2_1l, pt2_1r, pt1_1r)
  #face = entities.add_curve(pt1_1l, pt2_1l, pt2_1r, pt1_1r, pt1_1l)
  # Material
  face.material = $layers_mat[0]
  face.back_material = $layers_mat[0]
  # Follow me
  face.followme path
  
if( $layer_n > 1 )
# Second layer
    vec.length = $wall_width.to_f - $layer_wid_1.to_f
    pt1_2l = polar( pt1, pt2, vec.length, pis2 )
    pt1_2r = polar( pt1, pt2, vec.length - $layer_wid_2.to_f, pis2 )
    pt2_2l = Geom::Point3d.new(pt1_2l.x, pt1_2l.y, pt1_2l.z + $height)
    pt2_2r = Geom::Point3d.new(pt1_2r.x, pt1_2r.y, pt1_2r.z + $height)
  face = entities.add_face(pt1_2l, pt2_2l, pt2_2r, pt1_2r)
  #face = entities.add_curve(pt1_2l, pt2_2l, pt2_2r, pt1_2r, pt1_2l)
  face.material = $layers_mat[1]
  face.back_material = $layers_mat[1]
  # follow me
  face.followme path
end

if( $layer_n > 2 )
# Third layer
    vec.length = $wall_width.to_f - $layer_wid_1.to_f - $layer_wid_2.to_f
    pt1_3l = polar( pt1, pt2, vec.length, pis2 )
    pt1_3r = polar( pt1, pt2, vec.length - $layer_wid_3.to_f, pis2 )
    pt2_3l = Geom::Point3d.new(pt1_3l.x, pt1_3l.y, pt1_3l.z + $height)
    pt2_3r = Geom::Point3d.new(pt1_3r.x, pt1_3r.y, pt1_3r.z + $height)
  face = entities.add_face(pt1_3l, pt2_3l, pt2_3r, pt1_3r)
  #face = entities.add_curve(pt1_3l, pt2_3l, pt2_3r, pt1_3r, pt1_3l)
  face.material = $layers_mat[2]
  face.back_material = $layers_mat[2]
  # follow me
  face.followme path
end

if( $layer_n > 3 )
# Fourth layer
    vec.length = $wall_width.to_f - $layer_wid_1.to_f - $layer_wid_2.to_f - $layer_wid_3.to_f
    pt1_4l = polar( pt1, pt2, vec.length, pis2 )
    pt1_4r = polar( pt1, pt2, vec.length - $layer_wid_4.to_f, pis2 )
    pt2_4l = Geom::Point3d.new(pt1_4l.x, pt1_4l.y, pt1_4l.z + $height)
    pt2_4r = Geom::Point3d.new(pt1_4r.x, pt1_4r.y, pt1_4r.z + $height)
  face = entities.add_face(pt1_4l, pt2_4l, pt2_4r, pt1_4r)
  #face = entities.add_curve(pt1_4l, pt2_4l, pt2_4r, pt1_4r, pt1_4l)
  face.material = $layers_mat[3]
  face.back_material = $layers_mat[3]
  # follow me
  face.followme path
end

model.commit_operation
end    # of def

def hole_in_multiwalls
  model = Sketchup.active_model
  entities = model.active_entities
  ss = model.selection
  model.start_operation "Lines/curves to multiwalls"
  # Selection error checking
  if ss.empty? or (ss.length == 1) or (ss.length > 2)
    UI.messagebox("You must select 2 objects: a group and a face.")
    return nil
    else
      if(( ss[0].typename == "Group") and ( ss[1].typename == "Face")) or (( ss[0].typename == "Face") and ( ss[1].typename == "Group"))
      a = 1
      else
        UI.messagebox("These 2 objects are not of the correct type.")
        return nil
      end
  end
  
  #Who's the group and who's the face ?
  if( ss[0].typename == "Group")
    g = ss[0]
    f = ss[1]
  else
    g = ss[1]
    f = ss[0]
  end
  
end # of def

if( not file_loaded?("lines2multiwalls.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
    
end

#-----------------------------------------------------------------------------
file_loaded("lines2multiwalls.rb")

#        Name : lines2walls
# Description : Creates walls starting from a selection of lines, an arc, a circle, a curve
#      Author : Didier Bur
#       Usage : select lines, an arc, a circle, a curve, select from the Plugins menu, enter height and width and here you go
#        Date : 10.Aug.2oo4
#        Type : tool

require 'sketchup.rb'

def line_to_wall

model = Sketchup.active_model
model.start_operation "Line2wall"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

#ss.each |element|
# if not element.kind_of? Sketchup::Edge
#    UI.messagebox("Please select only lines.")
#    return nil
#  end
#end

prompts = ["Walls width", "Walls height "]
value = [25.cm,250.cm]
results = inputbox prompts, value, "Walls parameters"

return if not results # This means that the user canceld the operation
width, height = results


pis2 = 1.5707963267948965

# construct the wall profile
pts = ss[0].vertices
pt1 = pts[0].position
pt2 = pts[1].position
vec = pt1.vector_to pt2
rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], pis2))
pt1_left = pt1.offset(rotated_vec, width / 2)
rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - pis2)))
pt1_right = pt1.offset(rotated_vec, width / 2)
pt2_left = pt1_left.offset([0,0,1], height)
pt2_right = pt1_right.offset([0,0,1], height)
face = Sketchup.active_model.active_entities.add_face(pt1_left, pt2_left, pt2_right, pt1_right)

i = 0
path = []

0.upto(ss.length - 1) do |something|
  path[i] = ss[i]
  i = i + 1
end

# follow me along lines
face.reverse!.followme path

model.commit_operation
end    # of def


if( not file_loaded?("lines2walls.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
  
end

#-----------------------------------------------------------------------------
file_loaded("lines2walls.rb")

# ---------------------------------------------------------------------------
# Menu items:
#----------------------------------------------------------------------------
    wall_menu = UI.menu("Plugins").add_submenu("Line To Wall")
    wall_menu.add_item("Line to Wall") { line_to_wall }
    wall_menu.add_separator()
    wall_menu.add_item("Lines/Curves To Multi Layer Walls") { line_to_multiwalls }
    wall_menu.add_separator()
    wall_menu.add_item("Hole In Multi Layer Walls") { hole_in_multiwalls }
    wall_menu.add_separator()
    wall_menu.add_item("Lines/ Curves To Walls") { line_to_wall }
    wall_menu.add_separator()


